'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const fs = require('fs');
const path = require('path');

const debug = require('debug')('ruo');
const moder = require('moder');
const _ = require('lodash');
const Waterline = require('waterline');
const pascalcase = require('uppercamelcase');
const promiseify = require('denodeify');

const config = require('./config');

var _require = require('./translate');

const isTest = _require.isTest;


const waterline = new Waterline();
const initialize = promiseify(waterline.initialize.bind(waterline));

module.exports = (() => {
  var _ref = _asyncToGenerator(function* () {
    var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    let modelConfig = _ref2.model;

    let models = {};
    let services = {};
    // load models
    try {
      const modelDir = path.join(config.target, 'model');
      fs.statSync(modelDir);
      moder(modelDir, {
        lazy: false,
        filter: isTest,
        init(model) {
          // FIXME: why modelConfig.defaults not working?
          waterline.loadCollection(Waterline.Collection.extend(_.merge({}, modelConfig.defaults, model)));
        }
      });
      for (let name in modelConfig.adapters) {
        modelConfig.adapters[name] = require(modelConfig.adapters[name]);
      }
      const ontology = yield initialize(modelConfig);
      models = _.reduce(ontology.collections, function (obj, model, name) {
        name = pascalcase(name);
        obj[name] = model;
        return obj;
      }, {});
      // TODO: just pick one of the model, any better way?
      const Model = models[Object.keys(models)[0]];
      models.query = promiseify(Model.query.bind(Model));
      debug('models', Object.keys(models));
    } catch (err) {
      if (err.code !== 'ENOENT') {
        throw err;
      }
    }

    // load services
    try {
      const serviceDir = path.join(config.target, 'service');
      fs.statSync(serviceDir);
      services = moder(serviceDir, {
        lazy: false,
        filter: isTest
      });
      debug('services', Object.keys(services));
    } catch (err) {
      if (err.code !== 'ENOENT') {
        throw err;
      }
    }

    return { models, services };
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();